#include "s.h"
#include "spline.c"
#include "splint.c"
#include "gpiberror.c"
#include "nrutil.c"
#include <unistd.h>
#include <string.h>
#include <time.h>
int cardid[max_gpib_addr][nchannels/100];
int is_init[max_gpib_addr];
char setupstring[max_gpib_addr][stringl] = {"","","","","","","","","","","","","","","","","","","","","","","","","","","","","",""};
double nplc;
char dummystr[] = "res";
char dummystrfres[] = "fres";
char cardcfg[stringl] = {""};


void set_setup(int i,char* s)
{
  strcpy (setupstring[i],s);
  return;
}

void set_debug(int i)
{
  debug = i;
  return;
}

int strcmp_nolength(char* s1, char* s2)
/*
Returns an integral value indicating the relationship between the strings:
A zero value indicates that both strings are equal up to the length of the 
shortest string. A value greater than zero indicates that the first character 
that does not match has a greater value in str1 than in str2; And a value 
less than zero indicates the opposite.
Thus it can be used as a direct replacement for strcmp().
*/
{
  int i = 0;
  int min,l2;
  min = strlen(s1);
  l2 = strlen(s2);
  if (l2 < min)
    min = l2;
  while (i < min)
    {
      if (s1[i] != s2[i])
	return s1[i] - s2[i];
      i++;
    }
  return 0;
}

void chomp(char *s) {
  while(*s && *s != '\n' && *s != '\r') 
    s++;
  *s = 0;
}

void parse_conf(char *s)
{
  int i,j = 0;
  int l = strlen(s);
  char buff[stringl];
  memset(buff,0,stringl);
  for (i = 0; i < l;i++)
    {
      if (s[i] != '"')
	buff[j++] = tolower(s[i]);
    }
  strcpy(s,buff);
}

int now()
{
  struct tm start = {0};
  time_t timer,st;
  start.tm_year = 70;
  start.tm_mday = 1;
  st = mktime(&start);
  timer=time(NULL);
  return difftime(timer,st);
}


char* get_cards(int i)
{
  char scannerstring[stringl];
  int stl;
  bd = voltmeter2700[i];
  sprintf (scannerstring, "*opt?");
  stl = strlen (scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  strcpy (cardcfg, "");
  ibrd (bd, cardcfg, stringl);
  if (ibsta & ERR)
    GpibError ("ibrd Error");
  stl = strlen(cardcfg);
  if (cardcfg[stl-1] == '\n')
    cardcfg[stl-1] = 0;
  if (debug)
      printf("Card identifier string for GPIB %i: '%s'\n",i,cardcfg);
  return cardcfg;
}

void load_arr(int gpid)
{
  char* input = get_cards(gpid);
  int i;
  for (i=0;i!=nchannels/100;i++)
    {
      cardid[gpid][i]=0;
    }
  static char str[100];
  strcpy(str,input);
  char* tok = strtok(str,",");
  i = 0;
  while(tok != NULL)
    {
      if (strcmp("NONE",tok) != 0 )
	{
	  cardid[gpid][i] = atoi(tok);
	}
      tok = strtok(NULL,",");
      i++;
    }
  return;
}

int check_card_ch(int i, int ch)
{
  int card;
  card = ch / 100;
  int t = now();
  if (is_init[i] < t - updateinterval)
    {
      load_arr(i);
      is_init[i] = t;
    }
  return cardid[i][card-1];
}


void initgpib ()
{

  int i,j;
  for (i = 1; i != max_gpib_addr; ++i)
    {
      voltmeter2700[i] = gpibid[i];
      for (j=0;j!=nchannels/100;++j)
        {
          cardid[j][i] = 0;
        }
      is_init[i] = 0;
    }
}

void initchannels()
{
  int i, j, n, d;
  double rx, ry;
  char readres;
  char tempstring[stringl];
  FILE *channelsetupdata = NULL;
  char filename[stringl], STR1[stringl];
  char defaultfn[stringl]; 
  sprintf (defaultfn, "/etc/gpib/default.cal");
  for (j = 1; j != max_gpib_addr; ++j)
    {
      /* Reset card info array */
      for (i=0;i!=nchannels/100;++i)
        {
          cardid[j][i] = 0;
        }
      is_init[j] = 0;
      for (i = 101; i <= nchannels; i++)
	{
	  d = 0;
	  sprintf (filename, "%i/%d", j,i);
	  strcat (filename, ".cal");	  
	  sprintf (filename, "/etc/gpib/%s", strcpy (STR1, filename));
	  if (debug==1 || verbose_flag==1)
	    {
	      printf ("%s \n", filename);	  
	    }
	  channelsetupdata = fopen (filename, "r");
	  if (!channelsetupdata)
	    {
	      channelsetupdata = fopen (defaultfn, "r");
	      if (!channelsetupdata)
		{
		  printf("Could not open file %s!\n",defaultfn);
		  exit (1);
		}
	      d = 1;
	    }
	  fgets (tempstring, stringl, channelsetupdata);
	  /*          sprintf (channeldata[i].measuretype,"%s", tempstring);	  */
	  memset (channeldata[j][i].measuretype, '\0', stringl);

	  strncpy (channeldata[j][i].measuretype, tempstring,
		   strlen (tempstring) - 1);
	  memset (channeldata[j][i].ident, '\0', stringl);	  
	  fgets (tempstring, stringl, channelsetupdata);
	  if (d)
	    {
	      sprintf(tempstring, "Channel %d\n",i);
	    }
	  strncpy (channeldata[j][i].ident, tempstring, strlen (tempstring) - 1);
	  if (debug==1 || verbose_flag==1)
	    {
	      printf (" %s %s \n", channeldata[j][i].measuretype, channeldata[j][i].ident);
	    }
	  n = 0;
	  do
	    {
	      fscanf (channelsetupdata, "%lg%lg%*[^\n]", &rx, &ry);
	      readres = getc (channelsetupdata);
	      if (!(readres == EOF))
		{
		  n++;
		  channeldata[j][i].xspline[n] = rx;
		  channeldata[j][i].yspline[n] = ry;
		  if (debug==1 || verbose_flag==1)
		    {
		      printf (" %d %d %lg %lg \n", i, n, rx, ry);
		    }
		}
	    }
	  while (!(readres == EOF));
	  channeldata[j][i].ndata = n;
	  spline (channeldata[j][i].xspline, channeldata[j][i].yspline,
		  channeldata[j][i].ndata, 1e30, 1e30, channeldata[j][i].y2spline);
	  if (channelsetupdata)
	    fclose (channelsetupdata);
	  channelsetupdata = NULL;
	}
    }
}

void close_string_channel_2700(int i,int ch,char* conf,char* scannerstring)
{
  if (temp_flag)
    {
      sprintf (scannerstring, ":route:clos (@%3d)", ch);
      if (debug==1 || verbose_flag==1)
	printf("Setting channel definitions for GPIB %i: with config '%s' to '%s'\n",i,conf,scannerstring);
      return;
    }
  int card = ch/100;
  switch (cardid[i][card-1])
    {
    case 7700:
      sprintf (scannerstring, ":route:mult:clos (@%3d,%1d25)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:clos (@%3d,%3d,%1d23,%1d24,%1d25)", ch,(ch + 10),card,card,card);
      break;
    case 7702:
      sprintf (scannerstring, ":route:mult:clos (@%3d,%1d45)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:clos (@%3d,%3d,%1d43,%1d44,%1d45)", ch,(ch + 20),card,card,card);
      break;
    case 7703:
      sprintf (scannerstring, ":route:mult:clos (@%3d,%1d33)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:clos (@%3d,%3d,%1d33,%1d34,%1d35)", ch,(ch + 16),card,card,card);
      break;
    case 7708:
      sprintf (scannerstring, ":route:mult:clos (@%3d,%1d43)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:clos (@%3d,%3d,%1d41,%1d42,%1d43)", ch,(ch + 20),card,card,card);
      break;
    }
  if (debug==1 || verbose_flag==1)
    printf("Setting channel definitions for GPIB %i: with config '%s' to '%s'\n",i,conf,scannerstring);
  return;
}

void open_string_channel_2700(int i,int ch,char* conf,char* scannerstring)
{
  if (temp_flag)
    {
      sprintf (scannerstring, ":route:open:all");
      if (debug==1 || verbose_flag==1)
	printf("Setting channel definitions for GPIB %i: with config '%s' to '%s'\n",i,conf,scannerstring);
      return;
    }
  int card = ch/100;
  switch (cardid[i][card-1])
    {
    case 7700:
      sprintf (scannerstring, ":route:mult:open (@%3d,%1d25)", ch,card);	    
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:open (@%3d,%3d,%1d23,%1d24,%1d25)", ch,(ch + 10),card,card,card);
      break;
    case 7702:
      sprintf (scannerstring, ":route:mult:open (@%3d,%1d45)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:open (@%3d,%3d,%1d43,%1d44,%1d45)", ch,(ch + 20),card,card,card);
      break;
    case 7703:
      sprintf (scannerstring, ":route:mult:open (@%3d,%1d33)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:open (@%3d,%3d,%1d33,%1d34,%1d35)", ch,(ch + 16),card,card,card);
      break;
    case 7708:
      sprintf (scannerstring, ":route:mult:open (@%3d,%1d43)", ch,card);
      if (strcmp_nolength(conf, dummystrfres) == 0)
	sprintf (scannerstring, ":route:mult:open (@%3d,%3d,%1d41,%1d42,%1d43)", ch,(ch + 20),card,card,card);
      break;
    }
  if (debug==1 || verbose_flag==1)
    printf("Setting channel definitions for GPIB %i: with config '%s' to '%s'\n",i,conf,scannerstring);
  return;
}


double readchannel2700 (i, ch, rorg)
     int i;
     int ch;
     double *rorg;
{
  struct timespec tdel, trem;
  int stl, er, chorg;
  char readstring[stringl];
  char sendstring[stringl];
  char scannerstring[stringl];
  double rc, rconv;
  chorg = ch;
  bd = voltmeter2700[i];
  //  strcpy (sendstring, "*cls");
  //  ibwrt (bd, sendstring, strlen (sendstring));
  //  if (ibsta & ERR)
  //    GpibError ("ibwrt Error");
  tdel.tv_sec = 0;
  tdel.tv_nsec = 100 * 1000000; // 100 ms delay
  er = nanosleep (&tdel, &trem);
  if (!(strcmp (setupstring[i], channeldata[i][ch].measuretype) == 0))
    {
      strcpy (sendstring, "conf:");
      strcat (sendstring, channeldata[i][ch].measuretype);      
      ibwrt (bd, sendstring, strlen (sendstring));
      if (ibsta & ERR)
	GpibError ("ibwrt Error");
      strcpy (setupstring[i], channeldata[i][ch].measuretype);
      if (debug)
	printf("Changing config for for GPIB %i to '%s'\n",i,sendstring);
      sleep(1);
    }
  close_string_channel_2700(i,ch,setupstring[i],scannerstring);
  stl = strlen (scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  ibwrt (bd, ":read?", 6);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  strcpy (readstring, "");
  tdel.tv_sec = 0;
  tdel.tv_nsec = delay * 1000000;
  er = nanosleep (&tdel, &trem);
  ibrd (bd, readstring, stringl);
  if (ibsta & ERR)
    GpibError ("ibrd Error");
  rc = atof (readstring);
  if (debug)
    printf("Got result: %f\n",rc);
  if (rc > 1e30)
    {
      if (debug)
	printf("Retrying as result was too high!\n");
      ibwrt (bd, ":read?", 6);
      if (ibsta & ERR)
	GpibError ("ibwrt Error");
      strcpy (readstring, "");
      tdel.tv_sec = 0;
      tdel.tv_nsec = delay * 1000000;
      er = nanosleep (&tdel, &trem);
      ibrd (bd, readstring, stringl);
      if (ibsta & ERR)
	GpibError ("ibrd Error");
      rc = atof (readstring);
      if (debug)
	printf("Got result: %f\n",rc);
    }
  open_string_channel_2700(i,ch,setupstring[i],scannerstring);
  stl = strlen (scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  *rorg = rc;
  splint (channeldata[i][chorg].xspline, channeldata[i][chorg].yspline,
	  channeldata[i][chorg].y2spline, channeldata[i][chorg].ndata,
	  rc, &rconv);
  return rconv;
}


int burst2700_result ( i, ch )
     int i;
     int ch;
{
  struct timespec tdel, trem;
  int stl, er;
  char scannerstring[stringl];
  tdel.tv_sec = 0;
  tdel.tv_nsec = delay * 1000000;
  bd = voltmeter2700[i];
  if (verbose_flag)
    printf("Reading data.. \n");
  ibwrt (bd, "TRAC:DATA?", 10L);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  er = nanosleep (&tdel, &trem);
  static char longdata[longbuffer];
  memset (longdata,0,longbuffer);
  ibrd (bd, longdata, longbuffer);
  if (ibsta & ERR)
    GpibError ("ibrd Error");
  const char delimdata[] = ",";
  char *cp, *token;
  cp = strdup (longdata);
  token = strtok(cp,delimdata);
  int np = 0;
  while (token != NULL)
    {
      burst[np].raw = atof(token);
      token = strtok(NULL,delimdata);
      burst[np].time = atof(token);
      token = strtok(NULL,delimdata);
      token = strtok(NULL,delimdata);
      splint (channeldata[i][ch].xspline, channeldata[i][ch].yspline,
	      channeldata[i][ch].y2spline, channeldata[i][ch].ndata,  burst[np].raw, &burst[np].value);
      ++np;
    }
  sprintf (scannerstring, "INIT:CONT OFF");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "SENS:VOLT:DC:NPLC %.2f",nplc);
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  er = nanosleep (&tdel, &trem);
  open_string_channel_2700(i,ch,setupstring[i],scannerstring);
  stl = strlen (scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  return np;
}


void burst2700_init (i, ch, np)
     int i;
     int ch;
     int np;
{
  struct timespec tdel, trem;
  int stl, er;
  char readstring[stringl];
  char sendstring[stringl];
  char scannerstring[stringl];
  tdel.tv_sec = 0;
  tdel.tv_nsec = delay * 1000000;
  bd = voltmeter2700[i];
  //er = nanosleep (&tdel, &trem);
  if (!(strcmp (setupstring[i], channeldata[i][ch].measuretype) == 0))
    {

      strcpy (sendstring, "conf:");
      strcat (sendstring, channeldata[i][ch].measuretype);

      ibwrt (bd, sendstring, strlen (sendstring));
      if (ibsta & ERR)
	GpibError ("ibwrt Error");
      strcpy (setupstring[i], channeldata[i][ch].measuretype);
      if (debug==1 || verbose_flag==1)
	printf("Changing config for for GPIB %i to '%s'\n",i,sendstring);
      sleep(1);
    }
  /* Read old nplc value */
  sprintf (scannerstring, "SENS:VOLT:DC:NPLC?");
  if (verbose_flag)
    printf("Reading config: %s\n",scannerstring);
  stl = strlen (scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  er = nanosleep (&tdel, &trem);
  strcpy (readstring, "");
  ibrd (bd, readstring, stringl);
  nplc = atof(readstring);
  //er = nanosleep (&tdel, &trem);
  /* Set fast mode */
  sprintf (scannerstring, "SENS:VOLT:DC:NPLC 0.01");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  close_string_channel_2700(i,ch,setupstring[i],scannerstring);
  stl = strlen (scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "INIT:CONT ON");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "TRIG:DEL 0");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "TRAC:CLE");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "TRAC:TST:FORM ABS");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "TRAC:POIN %d", np);
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  sprintf (scannerstring, "TRAC:FEED SENS");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
  /* Start the measurements */
  sprintf (scannerstring, "TRAC:FEED:CONT NEXT");
  stl = strlen (scannerstring);
  if (verbose_flag)
    printf("Sending config: %s\n",scannerstring);
  ibwrt (bd, scannerstring, stl);
  if (ibsta & ERR)
    GpibError ("ibwrt Error");
  //  er = nanosleep (&tdel, &trem);
}


/* End. */
